"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SerialNumberInitializer = exports.NameInitializer = exports.FirmwareRevisionInitializer = exports.ModelInitializer = exports.ManufacturerInitializer = exports.VolumeInitializer = exports.TargetMediaStateInitializer = exports.MuteInitializer = exports.CurrentMediaStateInitializer = exports.BaseCharacteristicInitializer = void 0;
function isVolumeChange(command) {
    return command.command === "PUSH_VOLUME_COMMAND" /* PUSH_VOLUME_COMMAND */;
}
function isAudioPlayerState(command) {
    return command.command === "PUSH_AUDIO_PLAYER_STATE" /* PUSH_AUDIO_PLAYER_STATE */;
}
class BaseCharacteristicInitializer {
    constructor(logger, hap, alexa) {
        this.logger = logger;
        this.hap = hap;
        this.alexa = alexa;
    }
    setValue(service, value) {
        service.getCharacteristic(this.getCharacteristic()).setValue(value);
    }
    onSet(device, service, getCommand, getCommandValue) {
        this.alexa.onCharacteristicSet(device, service, this.getCharacteristic(), getCommand, getCommandValue);
    }
    onCommand(device, listener) {
        this.alexa.onDeviceCommand(device, listener);
    }
}
exports.BaseCharacteristicInitializer = BaseCharacteristicInitializer;
class CurrentMediaStateInitializer extends BaseCharacteristicInitializer {
    getCharacteristic() {
        return this.hap.Characteristic.CurrentMediaState;
    }
    initialize(service, device, media) {
        this.setValue(service, media.currentState);
        this.onCommand(device, command => {
            if (isAudioPlayerState(command)) {
                this.setValue(service, command.payload.audioPlayerState);
            }
        });
    }
    setValue(service, state) {
        const CurrentMediaState = this.hap.Characteristic.CurrentMediaState;
        let currentMediaState;
        switch (state) {
            case 'PLAYING':
                currentMediaState = CurrentMediaState.PLAY;
                break;
            case 'INTERRUPTED':
                currentMediaState = CurrentMediaState.PAUSE;
                break;
            case 'FINISHED':
            default:
                currentMediaState = CurrentMediaState.STOP;
                break;
        }
        super.setValue(service, currentMediaState);
    }
}
exports.CurrentMediaStateInitializer = CurrentMediaStateInitializer;
class MuteInitializer extends BaseCharacteristicInitializer {
    getCharacteristic() {
        return this.hap.Characteristic.Mute;
    }
    initialize(service, device, media) {
        this.setValue(service, media.muted);
        this.onSet(device, service, 'volume', value => value);
        this.onCommand(device, command => {
            if (isVolumeChange(command) && null !== command.payload.isMuted) {
                this.setValue(service, command.payload.isMuted);
            }
        });
    }
}
exports.MuteInitializer = MuteInitializer;
class TargetMediaStateInitializer extends BaseCharacteristicInitializer {
    getCharacteristic() {
        return this.hap.Characteristic.TargetMediaState;
    }
    initialize(service, device, media) {
        this.setValue(service, media.currentState);
        const TargetMediaState = this.hap.Characteristic.TargetMediaState;
        this.onSet(device, service, value => {
            switch (value) {
                case TargetMediaState.PAUSE:
                case TargetMediaState.STOP:
                    return 'pause';
                case TargetMediaState.PLAY:
                    return 'play';
            }
            throw new Error(`Unexpected value: ${JSON.stringify(value)}`);
        });
        this.onCommand(device, command => {
            if (isAudioPlayerState(command)) {
                this.setValue(service, command.payload.audioPlayerState);
            }
        });
    }
    setValue(service, state) {
        const TargetMediaState = this.hap.Characteristic.TargetMediaState;
        let targetMediaState;
        switch (state) {
            case 'PLAYING':
                targetMediaState = TargetMediaState.PLAY;
                break;
            case 'INTERRUPTED':
                targetMediaState = TargetMediaState.PAUSE;
                break;
            case 'FINISHED':
            default:
                targetMediaState = TargetMediaState.STOP;
                break;
        }
        super.setValue(service, targetMediaState);
    }
}
exports.TargetMediaStateInitializer = TargetMediaStateInitializer;
class VolumeInitializer extends BaseCharacteristicInitializer {
    getCharacteristic() {
        return this.hap.Characteristic.Volume;
    }
    initialize(service, device, media) {
        this.setValue(service, media.volume);
        this.onSet(device, service, 'volume', value => value);
        this.onCommand(device, command => {
            if (isVolumeChange(command) && null !== command.payload.volumeSetting) {
                this.setValue(service, command.payload.volumeSetting);
            }
        });
    }
}
exports.VolumeInitializer = VolumeInitializer;
class ManufacturerInitializer extends BaseCharacteristicInitializer {
    getCharacteristic() {
        return this.hap.Characteristic.Manufacturer;
    }
    initialize(service) {
        this.setValue(service, 'Amazon.com');
    }
}
exports.ManufacturerInitializer = ManufacturerInitializer;
class ModelInitializer extends BaseCharacteristicInitializer {
    getCharacteristic() {
        return this.hap.Characteristic.Model;
    }
    initialize(service, device) {
        this.setValue(service, device.deviceType);
    }
}
exports.ModelInitializer = ModelInitializer;
class FirmwareRevisionInitializer extends BaseCharacteristicInitializer {
    getCharacteristic() {
        return this.hap.Characteristic.FirmwareRevision;
    }
    initialize(service, device) {
        this.setValue(service, device.softwareVersion);
    }
}
exports.FirmwareRevisionInitializer = FirmwareRevisionInitializer;
class NameInitializer extends BaseCharacteristicInitializer {
    getCharacteristic() {
        return this.hap.Characteristic.Name;
    }
    initialize(service, device) {
        this.setValue(service, device.accountName);
    }
}
exports.NameInitializer = NameInitializer;
class SerialNumberInitializer extends BaseCharacteristicInitializer {
    getCharacteristic() {
        return this.hap.Characteristic.SerialNumber;
    }
    initialize(service, device) {
        this.setValue(service, device.serialNumber);
    }
}
exports.SerialNumberInitializer = SerialNumberInitializer;
//# sourceMappingURL=characteristicInitializer.js.map